"use strict";
// eslint-disable-next-line no-extend-native
Object.defineProperty(exports, "__esModule", { value: true });
exports.singleCall = exports.singleProcess = void 0;
function singleProcess(func) {
    let locker;
    return async function _singleProcess(...args) {
        await locker;
        locker = func.call(this, ...args);
        return locker;
    };
}
exports.singleProcess = singleProcess;
function singleCall(func) {
    const cache = {};
    return function _singleCall(...args) {
        const id = JSON.stringify(args);
        const cacheItem = cache[id];
        if (cacheItem) {
            if (cacheItem.error) {
                throw cacheItem.error;
            }
            return cacheItem.result;
        }
        if (cacheItem === false) {
            throw new Error(`Recursive call of single call func: ${func.toString()}`);
        }
        cache[id] = false;
        try {
            const result = func.call(this, ...args);
            cache[id] = { result };
            return result;
        }
        catch (error) {
            cache[id] = { error };
            throw error;
        }
    };
}
exports.singleCall = singleCall;
